/*===============================================================================
Title: dwYunWifi.ino
Description: deviceWISE Yun with WiFi Sample Program

Copyright (c)2015 Telit.
http://www.ilstechnology.com
http://www.devicewise.com


Legal Disclaimer:

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

    Neither the name of ILS Technology nor Telit nor the names of its
    contributors may be used to endorse or promote products derived
    from this software without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

===================================================================================*/


#include <dwArduinoYunWiFiOpenClient.h>  

// Authentication/Registration Details      
#define DWOPEN_APPTOKEN    "Xx7PIfDJQzhtVcdP"      //Application Token
#define DWOPEN_SERVER      "api.devicewise.com"    //deviceWISE server
#define DWOPEN_PORT        1883                    //MQTT Port

//Telit IoT Platform Headquarter - GPS Detail...
#define ILST_Latitude      "26.394919"
#define ILST_Longitude     "-80.112808"
#define ILST_Altitude      "3.0"
#define ILST_Speed         "0.0"
#define ILST_Heading       "0.0"


//Sensor I/O Definitions

#define RED_LED           4               //D4 - RED LED
#define BUZZER_PIN        2               //D2 - Buzzer
#define ANGLE_PIN         A0              //A0 - Angle/Rotary Sensor  


#define GREEN_ALARM         0             //Thing Green Alarm Value
#define YELLOW_ALARM        1             //Thing Yellow Alarm Value
#define RED_ALARM           2             //Thing Red Alarm Value

Process wifiCheck;                        //Yun WiFi process  
BridgeClient c;                           //Yun Bridge Support

char macAddr[30];                         //MAC Address Placeholder
char dwThingKey[40];                      //Placeholder for thing name

void callback(char* topic, uint8_t* payload, unsigned int length);
dwOpenClient dwClient((char*) DWOPEN_SERVER, DWOPEN_PORT, callback, &c);


//
// Yun Linux Shell function to obtain the MAC of the Yun WIFI
//

char *getMACAddress( char *MACBuffer, int BufLen )
{
  Bridge.begin();
  delay(2000);  // wait 2 seconds

  //Get our MAC Address

  //Run lua shell script to obtain MAC
  wifiCheck.runShellCommand(F("/usr/bin/pretty-wifi-info.lua | grep MAC"));  // command you want to run
  while(wifiCheck.running());   //Wait until the process finishes so you get the entire output
 
  String s = wifiCheck.readString();
  Serial.print(s);

  //Format the MAC Address string accordingly
  s = s.substring(13);
  s.replace("\n","");  
  s.replace(":","");
  
  s.toCharArray( MACBuffer, BufLen );  

  return( MACBuffer );
}


//
// Arduino standard setup() function - initialize all dwYunWifi resources
//
void setup() 
{
  int i;
    
  Serial.begin(9600);
  while( !Serial ) delay( 100 );  
  
  for(i = 5; i > 0; i-- )
  {
    Serial.print(F("Countdown="));
    Serial.println(i); 
    delay( 1000 );
  } 

  //Format the ThingKey for this run using our MAC
  sprintf(dwThingKey, "Yun-%s", getMACAddress(macAddr, sizeof(macAddr)));  
  Serial.print(F("\nMy Thing Key: "));Serial.println( dwThingKey );  


  pinMode(RED_LED, OUTPUT);
  pinMode(BUZZER_PIN, OUTPUT);  

  digitalWrite(BUZZER_PIN, LOW);  
}


//
// Arduino standard loop() function - ensure thing is connected to the cloud, publish parameters as required and set alarm value respectively
//
void loop() 
{
  int rc = -1;

  char szPropValue[16];
  static int LastRotary = -1;

  //Already connected to the cloud?
  if( !dwClient.connected() )
  {
      //Not connected, try connecting to the cloud.
      
      Serial.print(F("\nConnecting to: ")); Serial.print(DWOPEN_SERVER); Serial.print(F(" with client id: ")); Serial.println(dwThingKey);
  
      dwClient.connect((char*) dwThingKey, (char*) dwThingKey, (char*) DWOPEN_APPTOKEN);

      //Did we connect to the cloud?
      if( dwClient.connected() )
      {          
          //Publish Log Message... Connected
          Serial.println(F("\ndwYunWifi- Log Publish In Process...") );
          rc = dwClient.dwLogPublish( (char*) dwThingKey, "Arduino Yun IoT Application Started...");          
          dwClient.dwSleep(1000);

          //Publish Location Message...
          Serial.println(F("\ndwOpen API - Location Publish In Process..." ));      
          rc = dwClient.dwLocationPublish( dwThingKey, ILST_Latitude, ILST_Longitude, ILST_Altitude, ILST_Speed, ILST_Heading );            
          Serial.print(F("Publish Location return code : ")); Serial.println(rc);          
          dwClient.dwSleep(1000);                      

          //Publish Attribute Message...
          rc = dwClient.dwAttributePublish( dwThingKey, "MAC", macAddr );
          Serial.print(F("Publish Attribute return code : ")); Serial.println(rc);          
          dwClient.dwSleep(1000);         

          rc = dwClient.dwAlarmPublish( dwThingKey, "alarm1", GREEN_ALARM, "Init" );
          Serial.print(F("Publish Alarm return code : ")); Serial.println(rc);          
          dwClient.dwSleep(1000);           
      }    
       
  }

  //Are we connected to the cloud?
  if( dwClient.connected() )   
  {  
        //Yes, Read sensor value from potentiometer
        int RotaryValue = analogRead(ANGLE_PIN); 
  
        //Did the Rotary Value change?
        if (RotaryValue != LastRotary) 
        {
            //Yes, Publish new rotary value
            Serial.print(dwThingKey); Serial.print(F(", RotaryValue:")); Serial.println(RotaryValue);         
            sprintf(szPropValue,"%d", RotaryValue);             
            
            rc = dwClient.dwPropertyPublish( dwThingKey, "Rotary", szPropValue );    

            int AlarmStatus;
            if ( RotaryValue < 100 ) AlarmStatus = GREEN_ALARM;
            else if ( RotaryValue > 400 ) AlarmStatus = RED_ALARM;
            else AlarmStatus = YELLOW_ALARM;

            dwClient.dwAlarmPublish( dwThingKey, "alarm1", AlarmStatus, "" );
        }             
  }
  else
  {
  }

  //Sleep for 5 seconds
  dwClient.dwSleep(5000);
  
}


// Sets the LED accordingly
void setLED( int LEDPin, char *LEDStatus )
{
    if ( strcmp(LEDStatus, "1") == NULL )
    {        
        Serial.println("LED On!!!");          
        digitalWrite(LEDPin, HIGH);
    }
    else
    {
        Serial.println("LED Off!!!");          
        digitalWrite(LEDPin, LOW);          
    }  
}

// HandlesBuzzer Beep for Duration Value specified
void handleBuzzer( char *DurationValue )
{
  int Duration = atoi( DurationValue );

  Serial.print("in handleBuzzer: ");  Serial.println( DurationValue );   

  digitalWrite(BUZZER_PIN, HIGH);   
  delay(Duration);
  digitalWrite(BUZZER_PIN, LOW);   
}

//Extracts the value of the key specified
void ExtractKeyValue( char *haystack, char *Key, char *ResultValue )
{
  char *KeyPtr;
  char *KeyValue;
  char *KeyValueEnd;
  int   KeyValueLen;
  
  char needleTmp[40];
      
    strcpy( needleTmp, "\"");
    strcat( needleTmp, Key);
    strcat( needleTmp, "\":\"");

    KeyPtr = strstr(haystack, needleTmp );
      
    if( KeyPtr != NULL) 
    {   
        KeyValue = KeyPtr + strlen(needleTmp);
        KeyValueEnd = KeyValue;
    
        while( *KeyValueEnd != '\"')
        {
            KeyValueEnd++;
        }
    
        KeyValueLen = KeyValueEnd - KeyValue;
        strncpy( ResultValue, KeyValue, KeyValueLen );
        *(ResultValue + KeyValueLen ) = NULL;
    }
    else
    {
        strcpy( ResultValue, "" );
    }

}
        


//TR50 JSON Sample Request/Response
#define MAILBOX_IN_PROGRESS_ACK  "{\"999-999\": { \"command\": \"mailbox.check\", \"params\": { \"autoComplete\": true, \"limit\": 0 }}}"
#define MAILBOX_REQ_RECEIVED     "{\"0000123\": { \"command\": \"mailbox.update\", \"params\": {\"id\": \"%s\", \"msg\": \"Executing...\"}}}"
#define MAILBOX_REQ_ACK          "{\"0000124\": { \"command\": \"mailbox.ack\", \"params\": {\"id\": \"%s\", \"errorCode\": \"0\", \"errorMessage\": \"OK\"}}}"


// The function is called when an MQTT message arrives
void callback(char *topic, uint8_t *payload, unsigned int length) 
{
  int i;
  char jsonText[128]; 
  char msgID[25];
  char PayloadData[10];

  //Serial.println("Callback - MQTT Message Arrived...");
  //Serial.print("Topic: ");  Serial.println( topic ); 
  
  //Serial.print("Payload Length: ");  Serial.println( length ); 
  //Serial.print("Payload: ");  Serial.println( (char *) payload );
  
  if(strstr(topic, MQTT_NOTIFY) != NULL) 
  {  
      Serial.println(F("Notification - Mailbox Activity... Check Mailbox"));
      dwClient.publish_P("api", F(MAILBOX_IN_PROGRESS_ACK) );    
  }
  else if( (strstr(topic, MQTT_REPLY) != NULL) )
  {
      if ((strstr((char *)payload, "999-999") != NULL)) 
      {     
        Serial.println(F("Reply - Mailbox Check Response... Mailbox Request Received"));   
        ExtractKeyValue( (char *) payload, "id", msgID ); 
        
        sprintf_P( jsonText, (const char *) F(MAILBOX_REQ_RECEIVED), msgID );  
        dwClient.publish("api", jsonText );

        //Set LED accordingly                 
        ExtractKeyValue( (char *) payload, "duration", PayloadData );                
        if ( strlen(PayloadData) ) handleBuzzer(PayloadData);   
  
        ExtractKeyValue( (char *) payload, "varled", PayloadData );                
        if ( strlen(PayloadData) )  setLED( RED_LED, PayloadData);      
          
        Serial.println("Processing Complete - Send Mailbox Acknowledge...");
        sprintf_P( jsonText, (const char *) F(MAILBOX_REQ_ACK), msgID );   
        dwClient.publish( "api", jsonText );  
      } 
    
  }
  
}



